package net.sf.freecol.ai;

import java.io.IOException;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamReader;
import javax.xml.stream.XMLStreamWriter;

import net.sf.freecol.client.FreeColClient;
import net.sf.freecol.common.model.Colony;
import net.sf.freecol.common.model.FreeColObject;

import net.sf.freecol.common.model.Game;

import net.sf.freecol.common.model.PathNode;

import net.sf.freecol.common.model.Player;
import net.sf.freecol.common.model.Tile;
import net.sf.freecol.common.model.Unit;
import net.sf.freecol.common.model.Map.Direction;
import net.sf.freecol.common.model.Unit.MoveType;
import net.sf.freecol.common.networking.Message;

/**
 * This class represents the AI Player and has the 
 * responsibility of managing its actions and goals.
 * @author thiagolee
 * 
 * Nick
 * Making AIPlayer a subclass of Player, makes these two classes compairable and partially interchangeable
 *
 */

public class AIPlayer extends Player {
	
	/**
	 * The logger
	 */
	
	private static final Logger logger = Logger.getLogger(Player.class.getName());
	
	/**
     * The FreeColGameObject this object contains AI-information for.
     */
    private Player player;
    
    private Game game;
    
    
    /**
     * Also needed. The Client object.
     */
    private FreeColClient freeColClient;
    /**
     * Nick
     * admin, determines if the AIPlayer is admin of the game.
     */
    private boolean admin;
    
    private int nbrOfCalls = 0;

	public AIPlayer(Player player, FreeColClient freeColClient) {
		super();
		assignType();
		this.player = player;
		this.freeColClient = freeColClient;
		this.game = player.getGame();
	}
	
	public AIPlayer(Game game, String id) {
        super(game, id);
        assignType();
    }
	
	public AIPlayer(Game game, XMLStreamReader in) throws XMLStreamException {
        super(game, in);
        assignType();
    }
	
	public void assignType(){
		gameplayerType = PlayerTypes.AI;
	}
    
	/**
	 * Called every turn. From here, the AI decides what to do.
	 * @return true if everything went ok, false otherwise
	 */
	public boolean startWorking() {
		//buildFirstColony();
		System.out.println("AI: ending turn\n");
		this.endTurn();
		//goInDirection(10, Direction.N);
		//attackNearestColony();
		return true;
	}
	
	private void goInDirection(int nrMoves, Direction dir, List<Unit> myUnits) {
		for (Unit unit : myUnits) {
			//Move the unit for "nrMoves" steps to direction "dir"
			goInDirection(nrMoves, dir, unit);
			
		}
	}
	
	private void goInDirection(int nrMoves, Direction dir, Unit myUnit){
		while (nrMoves > 0) {
			System.out.println("nrMoves= " + nrMoves);
			if (myUnit.getMovesLeft() > 0) {
				MoveType myMoveType = myUnit.getMoveType(dir);
				if(myMoveType.equals(MoveType.MOVE_ILLEGAL))
					System.out.println("ended goInDirection, cannot move anymore");
				myUnit.move(dir);
				goInDirection(--nrMoves, dir, myUnit);

			}
			else{
				//no more moves left this turn, continue in the next turn!
				this.endTurn();
				goInDirection(nrMoves, dir, myUnit);
			}
			
		}
		System.out.println("nrMoves < 0");
		
	}
	private void goInDirection(int nrMoves, Direction dir){
		System.out.println("moving all units to Direction: " + dir);
		//use of this permits overriding of the getUnits method by the AIPlayer!
		List<Unit> myUnits = this.getUnits();
		System.out.println("Units: " + myUnits);
		System.out.println("isempty: " + myUnits.isEmpty());
		goInDirection(nrMoves, dir, myUnits);
		
		
	}
	private void attackNearestColony() {
		
		nbrOfCalls++;
		System.out.println("nrOfCalls:" + nbrOfCalls);
		Player me = getFreeColClient().getMyPlayer();
		
		List<Player> players = getFreeColClient().getGame().getPlayers();
		Tile tile = null;
		List<Colony> colonies = null;
		for (Player player : players) {
			if (player != me){
				colonies = player.getColonies();
				System.out.print("colonies found: ");
				System.out.println(colonies);
			}
			else
				continue;
			
			for (Colony colony : colonies) {
				tile = colony.getTile();
				System.out.println("Tile: " + colony.getName());
				break;
			}
			break;
		}		
		
		
		List<Unit> units = getFreeColClient().getMyPlayer().getUnits();
		
		for (Unit unit : units) {
			if (unit.isArmed()) {
				System.out.println("found armed unit");
				PathNode path = null; 
				if(tile != null)
					path = unit.findPath(unit.getTile(),tile);
				else System.out.println("no tile specified");
				System.out.println("found path");
				while(unit.getMovesLeft() > 0) {
					System.out.println("Moves left == " + unit.getMovesLeft());
					MoveType moveType = unit.getMoveType(path.getDirection());
					getFreeColClient().getInGameController().move(unit,
							path.getDirection());
					path = path.next;
				}
				System.out.println("break");											
			}
		}
		
		System.out.println("end");											
		me.endTurn();
		
	}

	/**
	 * Nick
	 * Test if the AIPlayer is the admin.	
	 * @return
	 */
	public boolean isAdmin(){
		return admin;
	}
	
	 /**
     * Gilles
     * Auto-build first colony
     */
    protected void buildFirstColony() {
    	/* we need to make sure it is our turn (checked outside function) */
    	this.moveWestUntilLand();
    	
        try {
        	if (player == getFreeColClient().getMyPlayer()) { // needed in case auto endofturn is active
        		getFreeColClient().getClient().getConnection().send(Message.createNewRootElement("endTurn"));
        	}
        } catch (IOException e) {
            logger.log(Level.WARNING, "Could not send \"endTurn\"-message!", e);
        }
    }
    
    protected void moveWestUntilLand() {
    	/* first check we're a naval unit */
    	Unit firstUnit = getFreeColClient().getGUI().getActiveUnit();
    	if(firstUnit.isNaval()) {
    		while(firstUnit.getMoveType(Direction.W).equals(MoveType.MOVE)) {
    			getFreeColClient().getInGameController().move(firstUnit, Direction.W);
    		}
    		if(firstUnit.getMoveType(Direction.W).equals(MoveType.DISEMBARK)) {
    			getFreeColClient().getInGameController().move(firstUnit, Direction.W);
    			
    		}
    		else {
    			if(firstUnit.getMoveType(Direction.W).equals(MoveType.MOVE_ILLEGAL)) {
    				/* unit cannot disembark there, move up */
    				getFreeColClient().getInGameController().move(firstUnit, Direction.N);
    			}
    		}
    	}
    	else {
    		/* Build colony */
    		if(firstUnit.canBuildColony()) {
    			getFreeColClient().getInGameController().buildColony();
    		}
    	}
    }
    
    @Override
	protected void toXMLImpl(XMLStreamWriter out) throws XMLStreamException {
		// TODO Auto-generated method stub
		
	}
    /**
     * Nick
     * Is called upon by InGameController
     */
    public void makeChoice(){
    	
    	
    }
    
    public Player getPlayer() {
		return player;
	}

	public FreeColClient getFreeColClient() {
		return freeColClient;
	}

}
