package net.sf.freecol.text;

import java.util.ArrayList;
import java.io.*;
import java.util.Collections;
import java.util.EnumMap;
import java.util.Iterator;
import java.util.List;
import java.util.Random;

public class Colony {
	
	public static int libertyPerColonist = 200;
	
	protected List<Location> tileList;
	protected List<Location> buildingList;
	protected List<Unit> unitList;
	protected List<Unit> militaryUnitList;
	protected List<Buildable> buildQueue;
	protected int[] buildOrder;
	protected int SoLPercentage;
	
	protected String name;
	protected Tile colonyTile;
	protected Location outside;
	
	protected int colonistIndex;
	
	protected EnumMap<Goods.Types, Integer> storage;
	protected int maxStorage;
	protected int maxFoodStorage;
	protected int prodModif;
	protected double defenseModif;
	
	//debug counters
	public int famine;
	
	public FileWriter fstream;
	public BufferedWriter stim;
	public BufferedWriter units;
	public BufferedWriter thresholds;
	public BufferedWriter synthetics;
	public BufferedWriter planning;
	
	//AI stuff
	protected double alpha;
	protected double beta;
	protected double gamma;
	protected double p = 0.2;
	
	public int plan = 0;
	
	protected EnumMap<Goods.Types, Integer> stimuli;
	public EnumMap<Goods.Types, Double> stimuliModifiers;
	
	public EnumMap<Goods.Types, Integer> planningStimuli;
	public EnumMap<Goods.Types, Double> planningStimuliModifiers;
	
	public EnumMap<Goods.Types, Integer> planningThresholds;
	
	public double alphaFood;
	public EnumMap<Goods.Types, Double> stimModFood;
	
	public double alphaProd;
	public EnumMap<Goods.Types, Double> stimModProd;
	
	private class Probability implements Comparable <Probability>{
		public Goods.Types type;
		public double prob;
		
		Probability(Goods.Types t, double d) {
			this.type = t;
			this.prob = d;
		}

		@Override
		public int compareTo(Probability p) {
			// TODO Auto-generated method stub
			return (int)(this.prob - p.prob);
		}
	}
	
	public Colony(String s, Tile tile) {
		initColony(s, tile);
	}
	
	public Colony(String s, Tile tile, EnumMap<Goods.Types, Double> map) {
		initColony(s, tile);
		this.stimuliModifiers = map;
		this.alphaFood = 1.0;
		this.stimModFood = map.clone();
		
		stimModFood.put(Goods.Types.FOOD, 0.084);
		stimModFood.put(Goods.Types.BELLS, 0.596);
		stimModFood.put(Goods.Types.NONE, 0.882);
		stimModFood.put(Goods.Types.LUMBER, 0.0);
		stimModFood.put(Goods.Types.HAMMERS, 0.0);
		stimModFood.put(Goods.Types.ORE, 0.0);
		stimModFood.put(Goods.Types.TOOLS, 0.0);
			
		this.alphaProd = 1.0;
		this.stimModProd = map.clone();
		stimModProd.put(Goods.Types.FOOD, 0.244);
		stimModProd.put(Goods.Types.BELLS, 0.669);
		stimModProd.put(Goods.Types.NONE, 1.896);
		stimModProd.put(Goods.Types.LUMBER, 0.468);
		stimModProd.put(Goods.Types.HAMMERS, 1.238);
		stimModProd.put(Goods.Types.ORE, 1.021);
		stimModProd.put(Goods.Types.TOOLS, 1.112);
	}
	
	public Colony(String s, Tile tile, EnumMap<Goods.Types, Double> map, double a, double b, double c) {
		initColony(s, tile);
		this.stimuliModifiers = map.clone();
		this.alpha = 1.0;
		this.beta = 9.984;
		this.gamma = 3.868;
		this.stimuliModifiers = map;
		this.alphaFood = 1.0;
		this.stimModFood = map.clone();
		stimModFood.put(Goods.Types.FOOD, 0.084);
		stimModFood.put(Goods.Types.BELLS, 0.596);
		stimModFood.put(Goods.Types.NONE, 0.882);
		stimModFood.put(Goods.Types.LUMBER, 0.0);
		stimModFood.put(Goods.Types.HAMMERS, 0.0);
		stimModFood.put(Goods.Types.ORE, 0.0);
		stimModFood.put(Goods.Types.TOOLS, 0.0);
			
		this.alphaProd = 1.0;
		this.stimModProd = map.clone();
		stimModProd.put(Goods.Types.FOOD, 0.244);
		stimModProd.put(Goods.Types.BELLS, 0.669);
		stimModProd.put(Goods.Types.NONE, 1.896);
		stimModProd.put(Goods.Types.LUMBER, 0.468);
		stimModProd.put(Goods.Types.HAMMERS, 1.238);
		stimModProd.put(Goods.Types.ORE, 1.021);
		stimModProd.put(Goods.Types.TOOLS, 1.112);
	}
	
	public void initColony(String s, Tile tile) {
		this.name = s;
		this.colonyTile = tile;
		this.outside = new OutsideColony();
		this.colonistIndex = 0;
		this.storage = new EnumMap<Goods.Types, Integer>(Goods.Types.class);
		this.tileList = new ArrayList<Location>();
		this.buildingList = new ArrayList<Location>();
		this.unitList = new ArrayList<Unit>();
		this.militaryUnitList = new ArrayList<Unit>();
		this.buildQueue = new ArrayList<Buildable>();
		this.buildOrder = new int[8];
		//init storage
		for(Goods.Types t: Goods.Types.values()) {
			if(t == Goods.Types.TOOLS) {
				storage.put(t, 100);
			}
			else {
				storage.put(t, new Integer(0));
			}
		}
		this.maxStorage = 100;
		this.maxFoodStorage = 200;
		this.prodModif = 0;
		this.defenseModif = 1.0;
		
		//create default buildings
		addDefaultBuildings();
		
		//debug: add default build list
		addDefaultBuildItems();
		
		//AI
		this.stimuli = new EnumMap<Goods.Types, Integer>(Goods.Types.class);
		this.stimuliModifiers = new EnumMap<Goods.Types, Double>(Goods.Types.class);
		this.planningStimuli = new EnumMap<Goods.Types, Integer>(Goods.Types.class);
		this.planningStimuliModifiers = new EnumMap<Goods.Types, Double>(Goods.Types.class);
		this.planningThresholds = new EnumMap<Goods.Types, Integer>(Goods.Types.class);
		//init stimuli
		for(Goods.Types t: Goods.Types.values()) {
			stimuli.put(t, new Integer(0));
			stimuliModifiers.put(t, new Double(1));
			//planning variables
			planningStimuli.put(t,  new Integer(0));
			planningStimuliModifiers.put(t, new Double(1.0));
			planningThresholds.put(t, new Integer(0));
		}
		this.setStimulus(Goods.Types.FOOD, 1);
		this.setStimulus(Goods.Types.BELLS, 1);
		this.setStimulus(Goods.Types.NONE, 1);
		this.setStimulus(Goods.Types.LUMBER, 1);
		this.setStimulus(Goods.Types.HAMMERS, 1);
		this.setStimulus(Goods.Types.ORE, 1);
		this.setStimulus(Goods.Types.TOOLS, 1);
		famine = 0;
		alpha = 1.0;
		beta = 1.0;
		gamma = 1.0;
		
		//write to file
		writeHeader1();
		writeHeader2();
		writeHeader3();
		writeHeader4();
		writeHeader5();
	}
	
	public void writeHeader1() {
		try {
			fstream = new FileWriter("stim.csv");
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		stim = new BufferedWriter(fstream);
		try {
			stim.write("FOOD,BELLS,NONE,HAMMERS,LUMBER,TOOLS,ORE\n");
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public void writeHeader2() {
		try {
			fstream = new FileWriter("units.csv");
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		units = new BufferedWriter(fstream);
		try {
			units.write("unit1, unit2, unit3\n");
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public void writeHeader3() {
		try {
			fstream = new FileWriter("synthetics.csv");
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		synthetics = new BufferedWriter(fstream);
		try {
			synthetics.write("SIZE, PRODUCTION MODIFIER, ARTILLERIES\n");
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public void writeHeader4() {
		try {
			fstream = new FileWriter("thresholds.csv");
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		thresholds = new BufferedWriter(fstream);
		try {
			thresholds.write("Unit, Expertise, plan, FOOD, BELLS, NONE, HAMMERS, LUMBER, TOOLS, ORE\n");
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public void writeHeader5() {
		try {
			fstream = new FileWriter("planning.csv");
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		planning = new BufferedWriter(fstream);
		try {
			planning.write("Plan\n");
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public void setStimulus(Goods.Types t, int i) {
		stimuli.put(t, i);
	}
	
	public int getStimulus(Goods.Types t) {
		return this.stimuli.get(t);
	}
		
	public void addDefaultBuildings() {
		Location townHall = new TownHall();
		this.buildingList.add(townHall);
		townHall.setColony(this);
		
		Location carpenterHouse = new CarpenterHouse();
		this.buildingList.add(carpenterHouse);
		carpenterHouse.setColony(this);
		
		Location blackSmith = new Blacksmith();
		this.buildingList.add(blackSmith);
		blackSmith.setColony(this);
	}
	
	public void addDefaultBuildItems() {
		/*
		Buildable LumberMill = new LumberMillBuildable();
		this.addToBuildList(LumberMill);
		
		Buildable printingPress = new PrintingPressBuildable();
		this.addToBuildList(printingPress);
		
		Buildable armory = new ArmoryBuildable();
		this.addToBuildList(armory);
		
		for(int i = 0; i < 50; ++i) {
			Buildable artillery = new ArtilleryBuildable();
			this.addToBuildList(artillery);
		}
		*/
	}
	
	public void setBuildQueue(int[] buildOrder) {
		
		System.arraycopy(buildOrder, 0, this.buildOrder, 0, buildOrder.length);
		
		buildQueue.clear();
		for(int i = 0; i < 50; ++i) {
			Buildable artillery = new ArtilleryBuildable();
			this.addToBuildList(artillery);
		}
		
		//add armory
		Buildable armory = new ArmoryBuildable();
		setInBuildList(buildOrder[0], armory);
		//add lumbermill
		Buildable LumberMill = new LumberMillBuildable();
		setInBuildList(buildOrder[1], LumberMill);
		//add blacksmith's shop
		Buildable blacksmithShop = new BlacksmithShopBuildable();
		setInBuildList(buildOrder[2], blacksmithShop);
		//add stockade
		Buildable stockade = new StockadeBuildable();
		setInBuildList(buildOrder[3], stockade);
		//add fort
		Buildable fort = new FortBuildable();
		setInBuildList(buildOrder[4], fort);
		//add warehouse
		Buildable warehouse = new WarehouseBuildable();
		setInBuildList(buildOrder[5], warehouse);
		//add printing press
		Buildable printpress = new PrintingPressBuildable();
		setInBuildList(buildOrder[6], printpress);
		//add newspaper
		Buildable newspaper = new NewsPaperBuildable();
		setInBuildList(buildOrder[7], newspaper);
	}
	
	public String getNewColonistName() {
		++this.colonistIndex;
		//System.out.println("New colonist is named "+this.name+"Colonist"+this.colonistIndex);
		return this.name+"Colonist"+this.colonistIndex;
	}
	
	public void setLocations(List<Location> l) {
		this.tileList = l;
	}
	
	public void addLocation(Location l) {
		if(!this.tileList.contains(l)) {
			this.tileList.add(l);
			l.setColony(this);
		}
	}
	
	public void addLocation(List<Location> l) {
		for(Location loc: l) {
			this.addLocation(loc);
		}
	}
	
	public void addUnit(Unit u) {
		if(!this.unitList.contains(u)) {
			this.unitList.add(u);
			u.setColony(this);
		}
	}
	
	public void removeUnitFromColony(Unit u) {
		if(unitList.contains(u)) {
			unitList.remove(u);
			u.setLocation(null);
		}
		else {
			System.out.println("This unit does not belong to this colony");
		}
	}
	
	public void setMaxStorage(int i) {
		this.maxStorage = i;
	}
	
	public List<Location> getTileList() {
		return this.tileList;
	}
	
	public List<Location> getBuildingList() {
		return this.buildingList;
	}
	
	public int getProductionModifier() {
		return this.prodModif;
	}
	
	public double computeProb1(Unit u, Goods.Types t, double stimulus) {
		double prob = 0.0;
		double stimulusSquare = this.stimuliModifiers.get(t)*stimulus*stimulus;
		double thresholdSquare = u.getThreshold(t)*u.getThreshold(t);
		prob = (stimulusSquare)/(stimulusSquare + alpha*thresholdSquare);
		return prob;
	}
	
	public double computePlanningProb(Goods.Types t, double stimulus) {
		double prob = 0.0;
		double stimulusSquare = this.planningStimuliModifiers.get(t)*stimulus*stimulus;
		double thresholdSquare = this.planningThresholds.get(t)*this.planningThresholds.get(t);
		prob = (stimulusSquare)/(stimulusSquare + thresholdSquare);
		return prob;
	}
	
	public List<Location> getLocList() {
		List<Location> locList = new ArrayList<Location>();
		locList.addAll(this.tileList);
		locList.addAll(this.buildingList);
		locList.add(this.outside);
		
		return locList;
	}
	
	public void addToBuildList(Buildable b) {
		b.col = this;
		this.buildQueue.add(b);
	}
	
	public void setInBuildList(int i, Buildable b) {
		b.col = this;
		this.buildQueue.set(i, b);
	}
	
	/**
	 * obsolete method
	 */
	public void updateStimuli() {
		//we need to make sure production modifier is up to date
		this.updateSoL();
		for(Goods.Types t: Goods.Types.values()) {
			switch(t) {
			case FOOD: simpleFoodStimulus();
				break;
			case BELLS: simpleBellStimulus();
				break;
			case NONE: //simpleOutsideStimulus(0); this has been integrated in the bells stimulus
				break;
			case LUMBER:
				break;
			case HAMMERS:
				break;
			case ORE:
				break;
			case TOOLS:
				break;
			default:
				break;
			}
		}
		
	}
	
	public void updateIncreasingStimuli(int size) {
		for(Goods.Types t: Goods.Types.values()) {
			switch(t) {
			case FOOD: simpleIncreasingFoodStim(size);
				break;
			case BELLS: simpleIncreasingBellStim(size);
				break;
			case NONE: simpleIncreasingOutsideStim(size);
				break;
			case LUMBER: simpleIncreasingLumberStim();
				break;
			case HAMMERS: simpleIncreasingHammerStim();
				break;
			case ORE: simpleIncreasingOreStim();
				break;
			case TOOLS: simpleIncreasingToolStim();
				break;
			default:
				break;
			}
		}
		//write stimuli to a file
		
		//writeStimuli1();
		
	}
	
	public void writeStimuli1() {
		try {
			for(Goods.Types t: Goods.Types.values()) {
				stim.write(this.getStimulus(t)+",");
			}
			stim.write(this.getSize()+","+this.prodModif);
			stim.flush();
			stim.newLine();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public void writePlanning() {
		try {
			planning.write(plan+",");
			planning.newLine();
			planning.flush();
			
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public void writeThresholds() {
		try {
			for(Unit u: this.unitList) {
				thresholds.write(u.getName()+","+u.expertise+","+plan+",");
				for(Goods.Types t: Goods.Types.values()) {
					thresholds.write(u.getThreshold(t)+",");
				}
				thresholds.flush();
				thresholds.newLine();
			}
			
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public void writeSynthetics() {
		try {
			synthetics.write(this.getSize()+","+this.prodModif+","+this.militaryUnitList.size());
			synthetics.flush();
			synthetics.newLine();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public void updatePlanningStimuli() {
		updateGrowthPlanStimulus();
		updateBuildPlanStimulus();
		updateLibertyPlanStimulus();
	}
	
	public void updateGrowthPlanStimulus() {
		planningStimuli.put(Goods.Types.FOOD, (int) beta);
		//set planning threshold = unitList.size()
		planningThresholds.put(Goods.Types.FOOD, (int) gamma *unitList.size());
		
	}
	
	public void updateBuildPlanStimulus() {
		planningStimuli.put(Goods.Types.HAMMERS, 5);
	}
	
	public void updateLibertyPlanStimulus() {
		planningStimuli.put(Goods.Types.BELLS, 5);
	}
	
	public void plan() {
		
		this.updatePlanningStimuli();
		//decide plan, for now just FOOD or PRODUCTION
		double prob = this.computePlanningProb(Goods.Types.FOOD, planningStimuli.get(Goods.Types.FOOD));
		double lottery = new Random().nextDouble();
		if(lottery <= prob) {
			//System.out.println("Planning should be FOOD");
			this.alpha = this.alphaFood;
			this.stimuliModifiers = this.stimModFood;
			plan = 1;
		}
		else {
			//System.out.println("Planning should be BUILD");
			this.alpha = this.alphaProd;
			this.stimuliModifiers = this.stimModProd;
			plan = 2;
		}
		//System.out.println("There's "+unitList.size()+" units");
		
	}
	
	public void updateFoodStimulus() {
		//for food, we want to avoid losing population at any cost
		//while maintaining moderate to high gross
		
		int foodConsumed = this.getSize()*2;
		int foodProduced = this.getFoodProduced();
		int foodSurplus = foodProduced - foodConsumed;
		
		//heuristic to guess what the food situation is going to be in 5 turns.
		int heuristic = this.storage.get(Goods.Types.FOOD)+(5*foodSurplus);
		//assert(heuristic > -25): "heuristic is really low. "+heuristic;
		//a low or negative heuristic should give a higher stimulus
		int stim = 11 - heuristic;
		stim = Math.max(stim, 0);
		this.setStimulus(Goods.Types.FOOD, stim);
		//System.out.println("FOOD stimulus is "+stim);
	}
	
	public void simpleFoodStimulus() {
		int foodConsumed = this.getSize()*2;
		int foodProduced = this.getFoodProduced();
		int foodSurplus = foodProduced - foodConsumed;
		
		if(foodSurplus < 0) {
			this.setStimulus(Goods.Types.FOOD, 10);
		}
		else {
			int futureGrowth = this.storage.get(Goods.Types.FOOD)+(5*foodSurplus);
			if(futureGrowth > 200) {
				//colony will grow, no need for more food stimulus
				this.setStimulus(Goods.Types.FOOD, 1);
			}
			else if (futureGrowth > 0) {
				this.setStimulus(Goods.Types.FOOD, 5);
			}
			else {
				//population will decrease, set max stimulus
				this.setStimulus(Goods.Types.FOOD, 10);
			}
		}
	}
	
	public void simpleIncreasingFoodStim2(int size) {
		int foodConsumed = size*2;
		int foodProduced = this.getFoodProduced();
		int foodSurplus = foodProduced - foodConsumed;
		
		int stim = this.stimuli.get(Goods.Types.FOOD);
		
		if(foodSurplus < 0) {
			Goods.Types highestStim = getHighestStim();
			this.setStimulus(Goods.Types.FOOD, stimuli.get(highestStim)+10);
		}
		this.setStimulus(Goods.Types.FOOD, Math.max(1, 10-foodSurplus));
	}
	
	public void simpleIncreasingFoodStim(int size) {
		int foodConsumed = size*2;
		int foodProduced = this.getFoodProduced();
		int foodSurplus = foodProduced - foodConsumed;
		
		int stim = this.stimuli.get(Goods.Types.FOOD);
		
		if(foodSurplus < 0) {
			this.setStimulus(Goods.Types.FOOD, 100);
			//this.setStimulus(Goods.Types.FOOD, stim + 25);
		}
		else if(foodSurplus < 6){
			this.setStimulus(Goods.Types.FOOD, stim + 1);
			//this.setStimulus(Goods.Types.FOOD, 5);
		}
		else {//reset stimulus to 0 as it has been attended to
			this.setStimulus(Goods.Types.FOOD, 1);
		}
	}	
	
	
	public void updateBellStimulus() {
		//for bells, we want to avoid production penalties
		int rebels = this.getSize() * this.SoLPercentage/100;
		int tories = this.getSize() - rebels; 
		//stimulus is in line with the gap between tories and freedom.
		int freedom = rebels - tories; //could be from -15 to +15 (roughly)
		
		int bellsProduced = this.getBellsProduced();
		int bellsEaten = Math.max(this.getSize()-2, 0);
		int bellSurplus = bellsProduced - bellsEaten; //could be from -15 to +15 (roughly)
		
		/**
		 * this probably should be analyzed/improved
		 */
		int stim = 10-freedom-bellSurplus;
		stim = Math.max(stim, 0);
		this.setStimulus(Goods.Types.BELLS, stim);
		//System.out.println("BELLS stimulus is "+stim);
	}
	
	public void simpleBellStimulus() {
		int bells = this.getBellsProduced();
		int bellsEaten = Math.max(this.getSize()-2, 0);
		int bellSurplus = bells - bellsEaten;
		if(bellSurplus < 0) {
			this.setStimulus(Goods.Types.BELLS, 10);
		}
		else {
			//we just check what would be the prod modifier with the next population growth
			//compared to what the prodMod is right now
			int futureSoL = ((this.storage.get(Goods.Types.BELLS)+5*bellSurplus)*100) / (libertyPerColonist *(this.getSize()+1));
			int futureProdMod = calculateProdMod(futureSoL);
			int prodEvolution = futureProdMod - this.prodModif; // {-1, 0, 1}
			switch(prodEvolution) {
			case -1: this.setStimulus(Goods.Types.BELLS, 10);
				break;
			case 0: this.setStimulus(Goods.Types.BELLS, 5);
				break;
			case 1: this.setStimulus(Goods.Types.BELLS, 1);
				break;
			}
			this.simpleOutsideStimulus(futureSoL);
		}
	}
	
	public void simpleIncreasingBellStim2(int size) {
		int bells = this.getBellsProduced();
		int bellsEaten = Math.max(size-2, 0);
		int bellSurplus = bells - bellsEaten;
		
		int stim = this.stimuli.get(Goods.Types.BELLS);
		/**
		 * remove when/if other means of producing bells are introduced
		 */
		if(!this.getBuildingByName("TOWNHALL").isFree()) {
			this.setStimulus(Goods.Types.BELLS, 0);
		}
		else {
			if(bellSurplus < 0) {
				this.setStimulus(Goods.Types.BELLS, stim +5);
			}
			else if(bellSurplus < 10) {
				this.setStimulus(Goods.Types.BELLS, stim +1);
			}
			else this.setStimulus(Goods.Types.BELLS, 0);
		}
	}
	
	public void simpleIncreasingBellStim(int size) {
		int bells = this.getBellsProduced();
		int bellsEaten = Math.max(size-2, 0);
		int bellSurplus = bells - bellsEaten;
		
		if(getBuildingByName("TOWNHALL").isFree())
			this.setStimulus(Goods.Types.BELLS, Math.max(0, 15-bellSurplus));
		else this.setStimulus(Goods.Types.BELLS, 0);
	}
	
	public void updateOutsideStimulus() {
		
	}
	
	public void simpleOutsideStimulus(int SoL) {
		int futureProdMod = calculateProdMod(SoL);
		//check if one additional unit will change the production modifier
		switch(futureProdMod) {
		case -2: this.setStimulus(Goods.Types.NONE, 10);
			break;
		case -1: this.setStimulus(Goods.Types.NONE, 5);
			break;
		case 0: 
		case 1: this.setStimulus(Goods.Types.NONE, 0);
			break;
		}	
	}
	
	public void simpleIncreasingOutsideStim(int size) {
		int stim = this.stimuli.get(Goods.Types.NONE);
		int SoL = computeSoLFromSize(size);
		int rebels = (size * SoL)/100;
		int tories = size -rebels;
		if(SoL <50 && (tories -rebels > 6)) {
			this.setStimulus(Goods.Types.NONE, stim + 5);
		}
		else {
			this.setStimulus(Goods.Types.NONE, 0);
		}		
	}
	
	public void simpleIncreasingLumberStim() {
		if(this.getHammerBuildQueue() > this.storage.get(Goods.Types.HAMMERS) && unitList.size() > 5 && storage.get(Goods.Types.LUMBER) < 80) {
		//if(this.getHammerBuildQueue() > this.storage.get(Goods.Types.HAMMERS) && storage.get(Goods.Types.LUMBER) < 80) {
			this.setStimulus(Goods.Types.LUMBER, 10);
		}
		else {
			this.setStimulus(Goods.Types.LUMBER, 0);
		}
	}
	
	public void simpleIncreasingLumberStim1() {
		int stim = this.stimuli.get(Goods.Types.LUMBER);
		int lumberSurplus = this.getLumberSurplus();
		if(lumberSurplus < 0 && storage.get(Goods.Types.LUMBER) < 50) {
			this.setStimulus(Goods.Types.LUMBER, stim+5);
			//this.setStimulus(Goods.Types.LUMBER, stim+10);
		}
		else this.setStimulus(Goods.Types.LUMBER, 0);
	}
	
	public void simpleIncreasingLumberStim2() {
		int stim = this.stimuli.get(Goods.Types.LUMBER);
		int lumberSurplus = this.getLumberSurplus();
		if(lumberSurplus < 0 && storage.get(Goods.Types.LUMBER) < 50) {
			this.setStimulus(Goods.Types.LUMBER, -lumberSurplus);
			//this.setStimulus(Goods.Types.LUMBER, stim+10);
		}
		else this.setStimulus(Goods.Types.LUMBER, 0);
	}
	
	public void simpleIncreasingOreStim1() {
		if(this.getHammerBuildQueue() > this.storage.get(Goods.Types.TOOLS) && unitList.size() > 5) {
			this.setStimulus(Goods.Types.ORE, 10);
		}
		else {
			this.setStimulus(Goods.Types.ORE, 0);
		}
	}
	
	public void simpleIncreasingOreStim3() {
		int stim = this.stimuli.get(Goods.Types.ORE);
		int oreSurplus = this.getOreSurplus();
		if(oreSurplus < 0 && storage.get(Goods.Types.ORE) < 50) {
			this.setStimulus(Goods.Types.ORE, stim+5);
			//this.setStimulus(Goods.Types.ORE, stim+10);
		}
		else this.setStimulus(Goods.Types.ORE, 0);
	}
	
	public void simpleIncreasingOreStim2() {
		int stim = this.stimuli.get(Goods.Types.ORE);
		int oreSurplus = this.getOreSurplus();
		if(oreSurplus < 0 && storage.get(Goods.Types.ORE) < 50) {
			this.setStimulus(Goods.Types.ORE, -oreSurplus);
			//this.setStimulus(Goods.Types.ORE, stim+10);
		}
		else this.setStimulus(Goods.Types.ORE, 0);
	}
	
	public void simpleIncreasingOreStim() {
		if(this.getToolBuildQueue() > this.storage.get(Goods.Types.TOOLS) && unitList.size() > 5 && storage.get(Goods.Types.TOOLS) < 100) {
		//if(this.getToolBuildQueue() > this.storage.get(Goods.Types.TOOLS) && storage.get(Goods.Types.ORE) < 80) {
			this.setStimulus(Goods.Types.ORE, 10);
		}
		else {
			this.setStimulus(Goods.Types.ORE, 0);
		}
	}
	
	public void simpleIncreasingHammerStim2() {
		int stim = this.stimuli.get(Goods.Types.HAMMERS);
		Location carpenterHouse  = this.getBuildingByName("CARPENTERHOUSE");
		if(!carpenterHouse.isFree()) {
			this.setStimulus(Goods.Types.HAMMERS, 0);
		}
		else {
			if(this.getHammerBuildQueue() > this.storage.get(Goods.Types.HAMMERS)) {
				int missingWorkers = 3 - carpenterHouse.occupancy;
				int maxProd = ((int) (3*carpenterHouse.upgradeMod)+this.prodModif)*missingWorkers;
				this.setStimulus(Goods.Types.HAMMERS, missingWorkers);
			}
			else this.setStimulus(Goods.Types.HAMMERS, 0);
		}
	}
	
	public void simpleIncreasingHammerStim() {
		int stim = this.stimuli.get(Goods.Types.HAMMERS);
		Location carpenterHouse  = this.getBuildingByName("CARPENTERHOUSE");
		if(storage.get(Goods.Types.LUMBER)>3*(3+prodModif)*carpenterHouse.upgradeMod) {
			if(!carpenterHouse.isFree()) {
				this.setStimulus(Goods.Types.HAMMERS, 0);
			}
			else {
				if(this.getHammerBuildQueue() > this.storage.get(Goods.Types.HAMMERS) && unitList.size() > 5) {
				//if(this.getHammerBuildQueue() > this.storage.get(Goods.Types.HAMMERS) ) {
					this.setStimulus(Goods.Types.HAMMERS, 10);
				}
				else this.setStimulus(Goods.Types.HAMMERS, 0);
			}
		}
		else {
			this.setStimulus(Goods.Types.HAMMERS, 0);
		}
	}
	
	public void simpleIncreasingToolStim1() {
		Location blacksmith  = this.getBuildingByName("BLACKSMITH");
		if(storage.get(Goods.Types.ORE)>3*(3+prodModif)*blacksmith.upgradeMod) {
			if(!blacksmith.isFree()) {
				this.setStimulus(Goods.Types.TOOLS, 0);
			}
			else {
				if(this.getToolBuildQueue() > this.storage.get(Goods.Types.TOOLS) && unitList.size() > 5) {
					this.setStimulus(Goods.Types.TOOLS, 10);
				}
				else this.setStimulus(Goods.Types.TOOLS, 0);
			}
		}
		else {
			this.setStimulus(Goods.Types.TOOLS, 0);
		}
	}
	
	public void simpleIncreasingToolStim2() {
		int stim = this.stimuli.get(Goods.Types.TOOLS);
		if(!this.getBuildingByName("BLACKSMITH").isFree()) {
			this.setStimulus(Goods.Types.TOOLS, 0);
		}
		else {
			if(this.getToolBuildQueue() > this.storage.get(Goods.Types.TOOLS)) {
				this.setStimulus(Goods.Types.TOOLS, 10);
			}
			else this.setStimulus(Goods.Types.TOOLS, 0);
		}
	}
	
	public void simpleIncreasingToolStim() {
		int stim = this.stimuli.get(Goods.Types.TOOLS);
		Location carpenterHouse  = this.getBuildingByName("BLACKSMITH");
		if(storage.get(Goods.Types.ORE)>3*(3+prodModif)*carpenterHouse.upgradeMod) {
			if(!carpenterHouse.isFree()) {
				this.setStimulus(Goods.Types.TOOLS, 0);
			}
			else {
				if(this.getToolBuildQueue() > this.storage.get(Goods.Types.TOOLS) && unitList.size() > 5) {
				//if(this.getToolBuildQueue() > this.storage.get(Goods.Types.TOOLS) ) {
					this.setStimulus(Goods.Types.TOOLS, 10);
				}
				else this.setStimulus(Goods.Types.TOOLS, 0);
			}
		}
		else {
			this.setStimulus(Goods.Types.TOOLS, 0);
		}
	}
	
	public int getHammerBuildQueue() {
		if(this.buildQueue.isEmpty()) {
			return 0;
		}
		else {
			return this.buildQueue.get(0).hammerCost;
		}
	}
	
	public int getToolBuildQueue() {
		if(this.buildQueue.isEmpty()) {
			return 0;
		}
		else {
			return this.buildQueue.get(0).toolCost;
		}
	}
	
	public int computeSoLFromSize(int size) {
		int SoL = this.storage.get(Goods.Types.BELLS)*100 / (libertyPerColonist *size);
		return Math.min(SoL, 100);
	}
	
	public void assignUnit(Unit u) {
		
		this.updateStimuli();
		
		//aggregate probabilities
		double maxProb = 0.0;
		List<Probability> probList = new ArrayList<Probability>();
		for(Goods.Types t: Goods.Types.values()) {
			switch(t) {
			case FOOD: maxProb += this.computeProb1(u, t, this.getStimulus(Goods.Types.FOOD));
					   probList.add(new Probability(t, maxProb));
				break;
			case BELLS: maxProb += this.computeProb1(u, t, this.getStimulus(Goods.Types.BELLS));
					   probList.add(new Probability(t, maxProb));
				break;
			case NONE: maxProb += this.computeProb1(u, t, this.getStimulus(Goods.Types.NONE));
			   		   probList.add(new Probability(t, maxProb)); 
				break;
			default : 
				break;
			}
		}
		//run the lottery
		double lottery = new Random().nextDouble()*maxProb;
		Collections.sort(probList);
		for(Probability p: probList) {
			if(lottery <= p.prob) {
				//unit will be assigned to role p.type unless there's no more room to produce that good
				Location l = this.findBestFreeLocForRole(p.type);
				//debug
				if(l.goodsOutput(p.type)== 0) {
					//System.out.println("This location doesn't produce any "+p.type);
					//should reassign unit according to another stimilus but there's a looping risk
				}
				//end debug
				u.setLocationAndRole(l, p.type);
				break;
			}
		}
	}
	
	public void assignAllUnits() {
		
		for(Unit u: this.unitList) {
			u.setLocation(outside);
		}
		
		for(Unit u: this.unitList) {
			this.assignUnit(u);
		}
	}
	
	public Location findBestFreeLocForRole(Goods.Types t) {
		
		int best = 0;
		Location loc = this.outside;
		List<Location> locList = new ArrayList<Location>();
		locList.addAll(this.tileList);
		locList.addAll(this.buildingList);
		locList.add(this.outside);
		
		for(Location l: locList) {
			if(l.isFree()) {
				int prod = l.goodsOutput(t);
				if(prod >= best) {
					loc = l;
					best = prod;
				}
			}
		}
		return loc;
	}
	
	public Location findBestFreeLocForRole2(Goods.Types t) {
		
		int best = 0;
		Location loc = null;
		
		if(t == Goods.Types.NONE) {
			loc = this.outside;
		}
		if(t == Goods.Types.BELLS) {
			loc = this.getBuildingByName("TOWNHALL");
		}
		if(t == Goods.Types.HAMMERS) {
			loc = this.getBuildingByName("CARPENTERHOUSE");
		}
		if(t == Goods.Types.TOOLS) {
			loc = this.getBuildingByName("BLACKSMITH");
		}
		
		if(loc != null && !loc.isFree()) {
			t = Goods.Types.FOOD;
			loc = null;
		}
		
		if(loc == null ) {
			for(Location l: this.tileList) {
				if(l.isFree()) {
					int prod = l.goodsOutput(t);
					if(prod >= best) {
						loc = l;
						best = prod;
					}
				}
			}
		}
		
		if(loc == null) {
			loc = this.outside;
		}
		
		return loc;
	}
	
	public void produce() {
		
		Goods.Types r = Goods.Types.FOOD;
		int store;
		
		for(Unit u : this.unitList) {
			//first round produces primary goods
			if(!u.getLocation().isFactory()) {
				r = u.getRole();
				store = this.storage.get(r);
				store += u.produceWithExperience();
				this.storage.put(r, store);
			}
		}
		
		for(Unit u : this.unitList) {
			//second round produces manufactured goods
			if(u.getLocation().isFactory()) {
				r = u.getRole();
				store = this.storage.get(r);
				store += u.produceWithExperience();
				this.storage.put(r, store);
			}
		}
		
		this.produceFromColonyTile();
				
	}
	
	public void build() {
		
		if(this.buildQueue.size() > 0) {
			//check if complete
			Buildable b = this.buildQueue.get(0);
			if(b.isComplete()) {
				b.buildItem();
				buildQueue.remove(b);
			}
		}
	}
	
	private void produceFromColonyTile() {
		int store = 0;
		Goods.Types r = null;
		
		//add production from colony tile.
		//This is food and the biggest secondary resource outside lumber
		store = this.storage.get(Goods.Types.FOOD);
		store += this.colonyTile.goodsOutput(Goods.Types.FOOD);
		this.storage.put(Goods.Types.FOOD, store);
		//secondary goods
		r = colonyTile.secondaryGoods();
		store = this.storage.get(r);
		store += this.colonyTile.goodsOutput(r);		
		this.storage.put(r, store);
		
		//Bells
		store = this.storage.get(Goods.Types.BELLS);
		++store;
		this.storage.put(Goods.Types.BELLS, store);
		
		//toDo CROSSES
	}
	
	public int maxColonyCapacity() {
		int max = 0;
		max += this.tileList.size();
		for(Location l: this.buildingList) {
			if(!l.getName().equals("OUTSIDE"))
			max += l.getMaxCapacity();
		}
		return max;
	}
	
	public Location findWorkLocation() {
		Location l = null;
		Goods.Types priority = Goods.Types.BELLS;
		
		for(Location loc : this.tileList) {
			if(loc.isFree()) {
				//give preference to food place
				l = loc;
				if(loc.produces() == Goods.Types.FOOD) return l;
			}
		}
		if(l==null || priority == Goods.Types.BELLS) {
			//if no food tile available then look for building
			for(Location b: this.buildingList) {
				if(b.isFree() && b.produces() == Goods.Types.BELLS) {
					l = b;
					return b;
				}
			}
		}
		if(l==null) {
			l = this.outside;
		}
		return l;
	}
	
	protected void updateStore(Goods.Types t) {
		int store = this.storage.get(t);
		switch(t) {
		case FOOD: 
			this.updateFood(store);
			break;
		case BELLS:
			this.updateSoL();
			break;
		case HAMMERS:
			this.updateHammers(store);
			break;
		default: 
			if(store > maxStorage) {
				//System.out.println("Warning: Storage of "+t+" has reached the limit of "+this.maxStorage);
				store = maxStorage;
			}
			this.storage.put(t, store);
			break;
		}
	}
	
	private void updateHammers(int store) {
		this.storage.put(Goods.Types.HAMMERS, store);
	}
	
	private void updateFood(int store) {
		if(store > maxFoodStorage) {
			//update storage and create a new colonist
			int surplus = store - maxFoodStorage;
			this.storage.put(Goods.Types.FOOD, surplus);
			if(this.unitList.size()==this.maxColonyCapacity()) {
				//System.out.println("This colony is full. Creating colonist outside");
				Unit u = new Unit(this.getNewColonistName(), this);
				//this.organizeColony();
				//this.assignAllUnits();
				//System.out.println(this);
			}
			else {
				//System.out.println("The colony grew!");
				Unit u = new Unit(this.getNewColonistName(), this);
				//this.organizeColony();
				//this.assignAllUnits();
				/*
				if(u.getLocation().isTile()) {
					u.setRole(Goods.Types.FOOD);
				}*/
				//this.reorganize();
				//System.out.println(this);
			}
		}
	}
	
	public void deleteUnit(Unit u) {
		//remove unit from colony and tile/building
		//u.getLocation().clearUnit(u);
		this.removeUnitFromColony(u);
	}
	
	public Unit getLastUnit() {
		//returns last unit of the 11 max
		int lastIndex = Math.min(this.unitList.size(), maxColonyCapacity());
		return this.unitList.get(lastIndex-1);
	}
	
	public void consumeFood() {
		int foodEaten = this.getSize()*2;
		int foodStored = this.storage.get(Goods.Types.FOOD);
		if(foodStored < foodEaten) {
			++famine;
			/*
			System.out.println("Warning: Colony does not produce enough food to sustain all colons");
			System.out.println(this.stimuli);
			System.out.println(this.storage);
			System.out.println(this.unitList);
			System.out.println("Food needed = "+foodEaten+" ,Bells surplus = "+getBellSurplus()+"\n");
			//System.out.println("DELETING unit "+ this.getLastUnit().getName());
			 * */
			this.deleteUnit(this.getLastUnit());
			this.organizeColony();
			//this.assignAllUnits();
			//System.out.println(this);
		}
		this.storage.put(Goods.Types.FOOD, Math.max(foodStored-foodEaten, 0));
	}
	
	public void consumeBells() {
		//cities consume 1 bell per colonist over two.
		int bellsEaten = Math.max(this.getSize()-2, 0);
		int bellsStored = Math.max(this.storage.get(Goods.Types.BELLS) - bellsEaten, 0);
		this.storage.put(Goods.Types.BELLS, bellsStored);
		
	}
	
	public void consumeLumber() {
		//this is handled when the unit produces hammer in the carpenterhouse
	}
	
	public void consume() {
		this.consumeFood();
		
		this.consumeBells();
		
		this.consumeLumber();
		
		for(Goods.Types t: Goods.Types.values()) {
			this.updateStore(t);
		}
	}
	
	public void showStorage() {
		int i;
		System.out.print("Storage for "+this.name+" : ");
		for(Goods.Types t: Goods.Types.values()) {
			if((i=this.storage.get(t)) != 0) {
				System.out.print(t+"="+i+" ,");
			}
		}
		System.out.println();
	}
	
	public Location getBuildingByName(String s) {
		Location loc = null;
		for(Location l : this.buildingList) {
			if(l.getName().equals(s)) {
				return l;
			}
		}
		return loc;
	}
	
	public void updateSoL() {
		this.SoLPercentage = this.storage.get(Goods.Types.BELLS)*100/(libertyPerColonist*this.getSize());
		//cap percentage at 100
		if (this.SoLPercentage > 100) {
			this.SoLPercentage = 100;
		}
		//System.out.println("SoL membership for colony "+this.name+" = "+SoLPercentage+"%");
		//check for production modifier
		int rebels = this.getSize() * this.SoLPercentage/100;
		int tories = this.getSize() - rebels; 
		
		if(this.SoLPercentage == 100) {
			this.prodModif = 2;
		}
		else if(this.SoLPercentage >= 50) {
			this.prodModif = 1;
		}
		else if(tories > 10) {
			//System.out.println("production modifier set to -2");
			this.prodModif = -2;
		}
		else if(tories > 6) {
			//System.out.println("production modifier set to -1");
			this.prodModif = -1;
		}
		else {
			this.prodModif = 0;
		}
	}
	
	public int calculateProdMod(int SoL) {
		int prodMod = 0;
		
		if (SoL > 100) {
			SoL = 100;
		}
		//System.out.println("SoL membership for colony "+this.name+" = "+SoLPercentage+"%");
		//check for production modifier
		int rebels = this.getSize() * SoL/100;
		int tories = this.getSize() - rebels; 
		
		if(SoL == 100) {
			prodMod = 2;
		}
		else if(SoL >= 50) {
			prodMod = 1;
		}
		else if(tories > 10) {
			//System.out.println("production modifier set to -2");
			prodMod = -2;
		}
		else if(tories > 6) {
			//System.out.println("production modifier set to -1");
			prodMod = -1;
		}
		else {
			prodMod = 0;
		}
		
		return prodMod;
	}
	
	public Goods.Types getHighestStim() {
		Goods.Types highest = Goods.Types.NONE;
		int max = 0;
		for(Goods.Types t: Goods.Types.values()) {
			if(stimuliModifiers.get(t)*getStimulus(t)>max) {
				max = getStimulus(t);
				highest = t;
			}
		}
		return highest;
	}
	
	public void organizeColony3() {
		for(Unit u: unitList) {
			u.setLocation(outside);
		}
		
		//greedy approach
		int size = this.unitList.size();
		Iterator<Unit> it = unitList.iterator();
		//fill up food
		while(getFoodSurplus() < 5 && it.hasNext()) {
			it.next().setLocationAndRole(findBestFreeLocForRole2(Goods.Types.FOOD), Goods.Types.FOOD);
		}
		//fill up bells
		while(this.getBuildingByName("TOWNHALL").isFree() && it.hasNext()) {
			it.next().setLocationAndRole(findBestFreeLocForRole2(Goods.Types.BELLS), Goods.Types.BELLS);
		}
		
		//fill up lumber
		if(it.hasNext() && prodModif >= 0) {
			it.next().setLocationAndRole(findBestFreeLocForRole2(Goods.Types.LUMBER), Goods.Types.LUMBER);
		}
		//fill up hammers
		while(this.getBuildingByName("CARPENTERHOUSE").isFree() && it.hasNext() && prodModif >= 0) {
			it.next().setLocationAndRole(findBestFreeLocForRole2(Goods.Types.HAMMERS), Goods.Types.HAMMERS);
		}
		//fill up ore
		if(it.hasNext() && prodModif >= 0) {
			it.next().setLocationAndRole(findBestFreeLocForRole2(Goods.Types.ORE), Goods.Types.ORE);
		}
		//fill up production
		while(this.getBuildingByName("BLACKSMITH").isFree() && it.hasNext() && prodModif >= 0) {
			it.next().setLocationAndRole(findBestFreeLocForRole2(Goods.Types.TOOLS), Goods.Types.TOOLS);
		}
	}
	
	public void organizeColony2() {
		//create list of available units
		List<Unit> availableUnits = new ArrayList<Unit>(unitList);
		
		//assign all units
		while (!availableUnits.isEmpty()) {
			//update stimuli
			this.updateIncreasingStimuli(this.getSize());
			//select highest stimuli
			Goods.Types currentStim = getHighestStim();
			//test all units
			double bestProb = 0.0;
			Unit bestUnit = null;
			for(Unit u: availableUnits) {
				double prob = this.computeProb1(u, currentStim, this.getStimulus(currentStim));
				double lottery = new Random().nextDouble();
				if(lottery < prob) {
					if(prob > bestProb) {
						bestProb = prob;
						bestUnit = u;
					}
				}
			}
			if(bestUnit != null) {
				//assign unit to best location and remove it from availableList
				bestUnit.setLocation(outside);
				Location bestLoc = this.findBestFreeLocForRole2(currentStim);
				bestUnit.setLocationAndRole(bestLoc, currentStim);
				availableUnits.remove(bestUnit);
				this.updateSoL();
			}
		}
		
	}
	
	public void organizeColony() {
		
		this.updateIncreasingStimuli(this.getSize());
		
		//check if there are workers that want to stop
		for(Unit u: unitList) {
			if(!u.loc.equals(outside)) {
				double lottery = new Random().nextDouble();
				if(lottery < p) {
					u.setLocation(outside);
				}
			}
		}
		
		//this.updateIncreasingStimuli(this.unitList.size());
		
		List<Unit>expertList = new ArrayList<Unit>();
		List<Unit>nonExpertList = new ArrayList<Unit>();
		//split in expert and non experts
		for(Unit u: this.unitList) {
			if(u.loc.equals(outside)) {
				boolean isExpert = (u.expertise == Goods.Types.NONE) ? nonExpertList.add(u) : expertList.add(u);
			}
		}
		
		List<Unit>randomUnitList = new ArrayList<Unit>(expertList);
		randomUnitList.addAll(nonExpertList);
		
		//assign/reassign units
		for(Unit u: randomUnitList) {
			//update stimuli
			/*
			if(u.loc == outside) {
				//reassigning unit inside would increase size by 1
				this.updateIncreasingStimuli(this.getSize()+1);
			}
			else {
				//unit is already working inside the colony so size is correct
				this.updateIncreasingStimuli(this.getSize());
			}
			*/
			//compute and test probs
			EnumMap<Goods.Types, Double> probMap = new EnumMap<Goods.Types, Double>(Goods.Types.class);
			List<Goods.Types> respondedToStim = new ArrayList<Goods.Types>();
			for(Goods.Types t: Goods.Types.values()) {
				probMap.put(t, this.computeProb1(u, t, this.getStimulus(t)));
				double lottery = new Random().nextDouble();
				if(lottery < probMap.get(t)) {
					respondedToStim.add(t);
				}
			}
			//Choose the stimuli with the highest stim/threshold ratio and assign location based on that
			Goods.Types bestStim = u.getRole();
			double bestProb = (double)this.getStimulus(bestStim)*this.stimuliModifiers.get(bestStim)/u.getThreshold(bestStim);;
			//randomize list to chose between stim of the same value fairly
			Collections.shuffle(respondedToStim);
			//assert(respondedToStim.isEmpty()):"did not respond to any stimulus";
			for(Goods.Types t: respondedToStim) {
				double stimToThreshold = (double)this.getStimulus(t)*this.stimuliModifiers.get(t)/u.getThreshold(t);
				if(stimToThreshold >= bestProb) {
					bestProb = stimToThreshold;
					bestStim = t;
				}
			}
			//finally, assign best location based on stimulus
			u.setLocation(outside);
			Location l = this.findBestFreeLocForRole2(bestStim);
			//Location currLoc = u.getLocation();
			//if(!u.getRole().equals(bestStim) || (u.getRole().equals(bestStim) && (l.goodsOutput(bestStim) >= currLoc.goodsOutput(bestStim)))) {
				u.setLocationAndRole(l, bestStim);
			//}
			//else {
				//don't change location
			//}
			this.updateSoL();			
		}
		
		//writeUnitRole1();
		
	}
	
	public void writeUnitRole1() {
		for(Unit u: this.unitList) {
			try {
				int i = roleToInt(u.role);
				String s = i+",";
				units.write(s, 0, s.length());
				units.flush();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		try {
			units.newLine();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public void writeUnitRole2() {
		for(Goods.Types t: Goods.Types.values()) {
			int i = 0;
			for(Unit u: this.unitList) {
				if(u.role.equals(t)) {
					++i;
				}
			}
			try {
				stim.write(i+",");
				stim.flush();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		try {
			stim.newLine();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public int roleToInt(Goods.Types t) {
		int i = 1;
		for(Goods.Types g: Goods.Types.values()) {
			if(g.equals(t)) break;
			++i;
		}
		return i;
	}
	
	public void reorganize() {
		//check food consumed with food produced
		this.updateSoL();
		int foodConsumed = this.getSize()*2;
		int foodProduced = this.getFoodProduced();
		
		if(foodConsumed > foodProduced) {
			System.out.println("reorganizing");
			//find unit not producing food
			boolean reassigned = false;
			for(Unit u: this.unitList) {
				if(reassigned) break;
				if(u.getRole() != Goods.Types.FOOD) {
					//look for a food location
					for(Location l : this.tileList) {
						if(l.isFree()) {
							u.setLocation(l);
							u.setRole(Goods.Types.FOOD);
							reassigned = true;
							break;
						}
					}
				}
			}
			if(!reassigned) System.out.println("Could not find a food tile to work");
		}
	}
	
	public int getFoodProduced() {
		int prod = 0;
		for(Unit u: this.unitList) {
			if(u.getRole()==Goods.Types.FOOD) prod += u.produce();
		}
		prod += this.colonyTile.goodsOutput(Goods.Types.FOOD);
		return prod;
	}
	
	public int getFoodSurplus() {
		int foodProduced = getFoodProduced();
		int foodConsumed = this.getSize()*2;
		return foodProduced - foodConsumed;
	}
	
	public int getBellsProduced() {
		int prod = 0;
		for(Unit u: this.getBuildingByName("TOWNHALL").unitList) {
			if(u.getRole()==Goods.Types.BELLS) prod += u.produce();
		}
		//Add bell produced by colony tile
		++prod;
		return prod;
	}
	
	public int getLumberProduced() {
		int prod = 0;
		for(Unit u: this.unitList) {
			if(u.getRole()==Goods.Types.LUMBER) prod += u.produce();
		}
		return prod;
	}
	
	public int getOreProduced() {
		int prod = 0;
		for(Unit u: this.unitList){
			if(u.getRole()==Goods.Types.ORE)prod += u.produce(); 
		}
		return prod;
	}
	
	public int getLumberSurplus() {
		int lumberProduced = this.getLumberProduced();
		Location l = this.getBuildingByName("CARPENTERHOUSE");
		int lumberConsumed = l.occupancy*(3*(int)(l.upgradeMod)+prodModif);
		return lumberProduced - lumberConsumed;
	}
	
	public int getOreSurplus() {
		int oreProduced = this.getOreProduced();
		int oreConsumed = this.getBuildingByName("BLACKSMITH").occupancy*(3+prodModif);
		return oreProduced - oreConsumed;
	}
	
	public int getBellSurplus() {
		int bellsProduced = getBellsProduced();
		int bellsConsumed = Math.max(this.getSize()-2, 0);
		return bellsProduced - bellsConsumed;
	}
	
	public int getSize() {
		int size = this.unitList.size()-this.outside.unitList.size()+this.militaryUnitList.size();
		if(size == 0) size = 2;
		return size;
		//can't have less than 1 unit
	}
	
	public int getScore() {
		//return this.unitList.size();
		//return this.SoLPercentage*this.militaryUnitList.size();
		//return this.SoLPercentage*this.unitList.size();
		//return this.SoLPercentage*this.storage.get(Goods.Types.HAMMERS);
		
		int SoLpoints = 0;
		if(SoLPercentage >= 50) {
			SoLpoints = 1;
		}
		return (int) (SoLpoints*this.militaryUnitList.size()*this.defenseModif);
		//return SoLpoints*this.unitList.size();
		//return storage.get(Goods.Types.BELLS);
		//return SoLpoints*this.storage.get(Goods.Types.HAMMERS);
		
	}
	
	public EnumMap<Goods.Types, Double> getModifiers() {
		return this.stimuliModifiers;
	}
	
	public double getAlpha() {
		return this.alpha;
	}
	
	public String toString() {
		String s = "Colony "+this.name+"\n";
		for(Unit u: this.unitList) {
			Location l = u.getLocation();
			if(l != outside && l != null) {
				String loc = l.getName();
				s += u.getName() +", "+u.expertise+" expert, "+ " in: "+loc+", producing "+u.getRole()+"\n";
			}
		}
		s += "There are "+this.militaryUnitList.size()+" artillery stationed\n";
		s += "SoL membership is "+this.SoLPercentage+"(Bells = "+this.storage.get(Goods.Types.BELLS)+" )\n";
		s += "Production modifier is "+this.prodModif+"\n";
		s += "Size "+this.getSize()+", outside: "+(this.outside.unitList.size()-this.militaryUnitList.size())+"\n";
		s += "Food surplus = "+getFoodSurplus()+" ,Bells surplus = "+getBellSurplus()+"\n";
		s += "Armory ["+buildOrder[0]+"], Lumbermill ["+buildOrder[1]+"], Blacksmith's shop ["+buildOrder[2]+
				"], Stockade ["+buildOrder[3]+"], Fort ["+buildOrder[4]+"] Warehouse ["+buildOrder[5]+
				"], Printing Press ["+buildOrder[6]+"], NewsPaper ["+buildOrder[7]+"]\n";
		return s;
	}
	
	public static void main(String[] args) {
		EnumMap<Goods.Types, Double> stimMod = new EnumMap<Goods.Types, Double>(Goods.Types.class);
		for(Goods.Types t: Goods.Types.values()) {
			stimMod.put(t, new Double(1));
		}
		int x = 0;
		/* best
		stimMod.put(Goods.Types.FOOD, 4.5);
		stimMod.put(Goods.Types.BELLS, 0.002);
		stimMod.put(Goods.Types.NONE, 1.57);
		*/
		
		
		// testing
		/*
		stimMod.put(Goods.Types.FOOD, 0.178);
		stimMod.put(Goods.Types.BELLS, 0.810);
		stimMod.put(Goods.Types.NONE, 1.060);
		stimMod.put(Goods.Types.LUMBER, 1.125);
		stimMod.put(Goods.Types.HAMMERS, 1.878);
		stimMod.put(Goods.Types.ORE, 1.861);
		stimMod.put(Goods.Types.TOOLS, 1.572);
		*/
		stimMod.put(Goods.Types.FOOD, 0.445);
		stimMod.put(Goods.Types.BELLS, 0.487);
		stimMod.put(Goods.Types.NONE, 1.009);
		stimMod.put(Goods.Types.LUMBER, 0.468);
		stimMod.put(Goods.Types.HAMMERS, 1.660);
		stimMod.put(Goods.Types.ORE, 0.580);
		stimMod.put(Goods.Types.TOOLS, 1.880);
		
		//int[] prodQueue = {0,1,21,5,7,27,2,31};
		int[] prodQueue = {1,0,4,5,7,45,2,3};
		//int[] prodQueue = {0,1,22,6,8,28,3,32};
		
		
		int score = 0;
		int size = 0;
		int famineCounter = 0;
		int military = 0;
		
		FileWriter file;
		BufferedWriter stats = null;
		try {
			file = new FileWriter("stats.csv");
			stats = new BufferedWriter(file);
			stats.write("SCORE, SIZE, FAMINE, MILITARY, SOL\n");
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		for(int i = 0; i < Game.iterations; ++i) {
			
			Colony c = new Colony("Brussels", new Prairie(false), stimMod, 1.0 , 0, 0);
			c.setBuildQueue(prodQueue);
			//c.beta = 4.0;
			//c.gamma = 1.0;
			//c.stimuliModifiers= c.stimModProd;
			//c.alpha = c.alphaProd;
			//Adding tiles
			c.addLocation(new Hill(true));
			c.addLocation(new Hill(false));
			c.addLocation(new MixedForest(true));
			c.addLocation(new MixedForest(false));
			c.addLocation(new Prairie(true));
			c.addLocation(new Prairie(false));
			c.addLocation(new Prairie(false));
			c.addLocation(new Plain(false));
			//Adding units
			for(int j = 0; j < 2; ++j) {
			Unit u1 = new Unit(c.getNewColonistName(), c);
			}
			
			
			for(int j = 0; j < Game.maxTurns; ++j) {
				//System.out.println(j);
				//if(j%5 == 0) c.plan();
				if(j%1 == 0) {
					//c.plan();
					c.organizeColony();
				}
				c.produce();
				c.consume();
				c.build();
				//System.out.println(c);
				c.writeSynthetics();
				c.writeThresholds();
				c.writeStimuli1();
				c.writeUnitRole1();
				c.writePlanning();
				
			}
			//System.out.println(c);
			score+=c.getScore();
			size+=c.unitList.size();
			military+=c.militaryUnitList.size();
			famineCounter += c.famine;
			System.out.println("Score ="+c.getScore()+" ,size = "+c.unitList.size()+" ,famine= "+c.famine+" ,military= "+c.militaryUnitList.size()+", SoL="+c.SoLPercentage);
			
			try {
				stats.write(c.getScore()+","+c.unitList.size()+","+c.famine+","+c.militaryUnitList.size()+","+c.SoLPercentage);
				stats.flush();
				stats.newLine();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			if(c.militaryUnitList.size() >= 24) break;
		}
		System.out.println("Score = "+score/Game.iterations+" ,Size = "+size/Game.iterations+" ,famine= "+(double)famineCounter/(double)Game.iterations+" ,military= "+military/Game.iterations);
		
	}

}
