package net.sf.freecol.text;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.EnumMap;
import java.util.HashSet;
import java.util.List;
import java.util.Random;
import java.util.Set;


public class Game {
	
	public static int generationSize = 200;
	public static int nrOfGeneration = 30;
	public static int nrOfParents = 100;
	public static int iterations = 100;
	public static int maxTurns = 460;
	public static double mutationChance = 0.01;
	public static double minCoeff = 0.0;
	public static double maxCoeff = 2.0;
	public static double minCoeff2 = 0.0;
	public static double maxCoeff2 = 10.0;
	
	public Gene best1;
	public Gene best2;
	
	public static int gene1Size = 7; //FOOD, BELLS, NONE, LUMBER, HAMMER, ORE, TOOLS
	public static int gene2Size = 8;
	public static int gene3Size = 2;
	
	private class Gene extends Thread implements Comparable <Gene>{
		
		public double alpha;
		public double beta;
		public double gamma;
		public EnumMap<Goods.Types, Double> stimuliModifiers;
		public int[] buildQueue;
		public int score;
		public int size;
		public int military;
		public double famine;
		public int lowest;
		public int highest;
		
		public double alphaFood;
		public EnumMap<Goods.Types, Double> stimModFood;
		
		public double alphaProd;
		public EnumMap<Goods.Types, Double> stimModProd;
		
		public Gene() {
			
		}
		
		public Gene(double a, EnumMap<Goods.Types, Double> map, int[] bQueue) {
			this.alpha = a;
			this.stimuliModifiers = map.clone();
			this.buildQueue = new int[] {0,1,22,6,8,28,3,32};
			//this.buildQueue = bQueue.clone();
			
			this.alphaFood = 1.0;
			this.stimModFood = map.clone();
			stimModFood.put(Goods.Types.FOOD, 0.211);
			stimModFood.put(Goods.Types.BELLS, 1.488);
			stimModFood.put(Goods.Types.NONE, 1.850);
			stimModFood.put(Goods.Types.LUMBER, 0.0);
			stimModFood.put(Goods.Types.HAMMERS, 0.0);
			stimModFood.put(Goods.Types.ORE, 0.0);
			stimModFood.put(Goods.Types.TOOLS, 0.0);
				
			this.alphaProd = 1.0;
			this.stimModProd = map.clone();
			stimModProd.put(Goods.Types.FOOD, 0.840);
			stimModProd.put(Goods.Types.BELLS, 0.929);
			stimModProd.put(Goods.Types.NONE, 1.621);
			stimModProd.put(Goods.Types.LUMBER, 0.360);
			stimModProd.put(Goods.Types.HAMMERS, 1.747);
			stimModProd.put(Goods.Types.ORE, 1.563);
			stimModProd.put(Goods.Types.TOOLS, 1.781);
				
			this.beta = randDouble(minCoeff2, maxCoeff2);
			this.gamma = randDouble(minCoeff2, maxCoeff2);
				
			//this.stimuliModifiers = stimModProd.clone();
		}
		
		public Gene(double a, EnumMap<Goods.Types, Double> map1, EnumMap<Goods.Types, Double> map2,EnumMap<Goods.Types, Double> map3,int[] bQueue) {
			this.alpha = a;
			this.stimuliModifiers = map1.clone();
			//this.buildQueue = new int[] {0,1,22,6,8,28,3,32};
			this.buildQueue = new int[] {1,0,4,5,7,45,2,3};
			//this.buildQueue = bQueue.clone();
			
			this.alphaFood = 1.0;
			this.stimModFood = map2.clone();
			stimModFood.put(Goods.Types.FOOD, 0.084);
			stimModFood.put(Goods.Types.BELLS, 0.596);
			stimModFood.put(Goods.Types.NONE, 0.882);
			stimModFood.put(Goods.Types.LUMBER, 0.0);
			stimModFood.put(Goods.Types.HAMMERS, 0.0);
			stimModFood.put(Goods.Types.ORE, 0.0);
			stimModFood.put(Goods.Types.TOOLS, 0.0);
				
			this.alphaProd = 1.0;
			this.stimModProd = map3.clone();
			stimModProd.put(Goods.Types.FOOD, 0.244);
			stimModProd.put(Goods.Types.BELLS, 0.669);
			stimModProd.put(Goods.Types.NONE, 1.896);
			stimModProd.put(Goods.Types.LUMBER, 0.468);
			stimModProd.put(Goods.Types.HAMMERS, 1.238);
			stimModProd.put(Goods.Types.ORE, 1.021);
			stimModProd.put(Goods.Types.TOOLS, 1.112);
			
			this.beta = randDouble(minCoeff2, maxCoeff2);
			this.gamma = randDouble(minCoeff2, maxCoeff2);
				
			//this.stimuliModifiers = stimModProd.clone();
		}
		
		public Gene(Gene g) {
			this.alpha = g.alpha;
			this.beta = g.beta;
			this.gamma = g.gamma;
			this.stimuliModifiers = g.stimuliModifiers.clone();
			this.stimModFood = g.stimModFood.clone();
			this.stimModProd = g.stimModProd.clone();
			this.buildQueue = g.buildQueue.clone();
		}
		
		public void run() {
			int score = 0;
			int size = 0;
			int military = 0;
			double famine = 0;
			int lowest = 100;
			int highest = 0;
			for(int i = 0; i<Game.iterations; ++i) {
				Colony c = buildScenario1(this.alpha, this.stimuliModifiers);
				c.setBuildQueue(buildQueue);
				c.beta = this.beta;
				c.gamma = this.gamma;
				c.stimModFood = this.stimModFood;
				c.stimModProd =this.stimModProd;
				//c.storage.put(Goods.Types.BELLS, 2000);
				//c.assignAllUnits();
				
				for(int j = 0; j < Game.maxTurns; ++j) {
					if(j%1 == 0) {
						//c.plan();
						c.organizeColony();
					}
					c.produce();
					c.consume();
					c.build();
					
				}
				lowest = Math.min(lowest, c.militaryUnitList.size());
				highest = Math.max(highest, c.militaryUnitList.size());
				size += c.unitList.size();
				score += c.getScore();
				famine += c.famine;
				military += c.militaryUnitList.size();
			}
			this.score = score/Game.iterations;
			this.famine = famine/(double)Game.iterations;
			this.size = size/Game.iterations;
			this.military = military/Game.iterations;
			this.lowest = lowest;
			this.highest = highest;
			//System.out.println(this);
		}
		
		public double get(int i) {
			double value = 0;
			switch(i) {
			case 0: value = this.alpha; break;
			case 1: value = this.stimuliModifiers.get(Goods.Types.FOOD); break;
			case 2: value = this.stimuliModifiers.get(Goods.Types.BELLS); break;
			case 3: value = this.stimuliModifiers.get(Goods.Types.NONE); break;
			case 4: value = this.stimuliModifiers.get(Goods.Types.HAMMERS); break;
			case 5: value = this.stimuliModifiers.get(Goods.Types.LUMBER); break;
			case 6: value = this.stimuliModifiers.get(Goods.Types.TOOLS); break;
			case 7: value = this.stimuliModifiers.get(Goods.Types.ORE); break;
			}
			return value;
		}
		
		public void set(int i, double d) {
			switch(i) {
			case 0: this.alpha = d; break;
			case 1: this.stimuliModifiers.put(Goods.Types.FOOD, d); break;
			case 2: this.stimuliModifiers.put(Goods.Types.BELLS, d); break;
			case 3: this.stimuliModifiers.put(Goods.Types.NONE, d); break;
			case 4: this.stimuliModifiers.put(Goods.Types.HAMMERS, d); break;
			case 5: this.stimuliModifiers.put(Goods.Types.LUMBER, d); break;
			case 6: this.stimuliModifiers.put(Goods.Types.TOOLS, d); break;
			case 7: this.stimuliModifiers.put(Goods.Types.ORE, d); break;
			}
		}
		
		public void set(int i, double d, EnumMap<Goods.Types, Double> map1) {
			switch(i) {
			case 0: this.alpha = d; break;
			case 1: map1.put(Goods.Types.FOOD, d); break;
			case 2: map1.put(Goods.Types.BELLS, d); break;
			case 3: map1.put(Goods.Types.NONE, d); break;
			case 4: map1.put(Goods.Types.HAMMERS, d); break;
			case 5: map1.put(Goods.Types.LUMBER, d); break;
			case 6: map1.put(Goods.Types.TOOLS, d); break;
			case 7: map1.put(Goods.Types.ORE, d); break;
			}
		}
		
		@Override
		public int compareTo(Gene g) {
			// TODO Auto-generated method stub
			return (int)(g.score - this.score);
		}
		
		public void correctBuildQueue() {
			Set<Integer> match = new HashSet<Integer>();
			for(int i = 0; i < buildQueue.length; ++i) {
				boolean conflict = match.add(buildQueue[i]);
				while(!conflict) {
					buildQueue[i]=(buildQueue[i]+1)%30;
					conflict = match.add(buildQueue[i]);
				}
			}
		}
		
		public String toString() {
			return "alpha: "+this.alpha+"\n"+" ,FOOD: "+this.stimuliModifiers.get(Goods.Types.FOOD)+"\n"+
					" ,BELLS: "+this.stimuliModifiers.get(Goods.Types.BELLS)+"\n"+" ,NONE: "+this.stimuliModifiers.get(Goods.Types.NONE)+"\n"+
					" ,LUMBERS: "+this.get(5)+"\n"+" ,HAMMERS: "+this.get(4)+"\n"+" ,ORE: "+this.get(7)+"\n"+" ,TOOLS: "+this.get(6)+"\n"+
					" , Score: "+this.score+" ,size: "+this.size+" ,military: "+this.military+" ,Famine: "+this.famine+"\n"+
					"Armory ["+buildQueue[0]+"], Lumbermill ["+buildQueue[1]+"], Blacksmith's shop ["+buildQueue[2]+
					"], Stockade ["+buildQueue[3]+"], Fort ["+buildQueue[4]+"] Warehouse ["+buildQueue[5]+
					"], Printing Press ["+buildQueue[6]+"], NewsPaper ["+buildQueue[7]+"]\n"+
					"Mod FOOD\n"+this.stimModFood+"\nMod PROD\n"+this.stimModProd+"\n"+
					"beta = "+this.beta+", gamma = "+this.gamma+", lowest = "+this.lowest+", highest = "+this.highest+"\n";
		}
	}
	
	protected List<Gene> currentGeneration;
	protected List<Gene> nextGeneration;
	
	public Game() {
		
		currentGeneration = new ArrayList<Gene>();
		nextGeneration = new ArrayList<Gene>();
		
		best1 = new Gene(1.0, nextStimMod(minCoeff, maxCoeff), randomBuildQueue());
		best2 = new Gene(1.0, nextStimMod(minCoeff, maxCoeff), randomBuildQueue());
			
		for(int i = 0; i<generationSize; ++i) {
			//Gene g = new Gene(1.0, nextStimMod(minCoeff, maxCoeff), randomBuildQueue());
			Gene g = new Gene(1.0, nextStimMod(minCoeff, maxCoeff), nextStimModFood(minCoeff, maxCoeff),nextStimMod(minCoeff, maxCoeff),randomBuildQueue());
			currentGeneration.add(g);
		}
	}
	
	public Colony buildScenario1(double alpha, EnumMap<Goods.Types, Double> map) {
		//this reconstructs the world as per scenario1b
		Colony c = new Colony("Brussels", new Prairie(false), map, alpha, 0, 0);
		//Adding tiles
		c.addLocation(new Hill(true));
		c.addLocation(new Hill(false));
		c.addLocation(new MixedForest(true));
		c.addLocation(new MixedForest(false));
		c.addLocation(new Prairie(true));
		c.addLocation(new Prairie(false));
		c.addLocation(new Prairie(false));
		c.addLocation(new Plain(false));
		
		//Adding units
		for(int i = 0; i < 2; ++i) {
			Unit u = new Unit(c.getNewColonistName(), c);
		}
		
		return c;
	}	
	
	public Colony buildScenario2(double alpha, EnumMap<Goods.Types, Double> map) {
		//this reconstructs the world as per scenario1b
		Colony c = new Colony("Brussels", new Prairie(false), map, alpha, 0, 0);
		//Adding tiles
		c.addLocation(new Hill(true));
		c.addLocation(new Hill(false));
		c.addLocation(new MixedForest(true));
		c.addLocation(new MixedForest(false));
		c.addLocation(new Prairie(true));
		c.addLocation(new Prairie(false));
		c.addLocation(new Prairie(false));
		c.addLocation(new Plain(false));
		
		//Adding units
		for(int i = 0; i < 8; ++i) {
			Unit u = new Unit(c.getNewColonistName(), c);
		}
		
		return c;
	}	
	
	public double randDouble(double min, double max) {
		double d;
		double diff = max - min;
		d = new Random().nextDouble()*diff;
		d = min + d;
		return d;
	}
	
	public EnumMap<Goods.Types, Double> nextStimMod(double min, double max) {
		EnumMap<Goods.Types, Double> stimMod =  new EnumMap<Goods.Types, Double>(Goods.Types.class);
		for(Goods.Types t : Goods.Types.values()) {
			if(t.equals(Goods.Types.FOOD) || t.equals(Goods.Types.BELLS) || t.equals(Goods.Types.NONE)) {
				stimMod.put(t, randDouble(min, max));
			}
			else {
				//stimMod.put(t, 0.0);
				stimMod.put(t, randDouble(min, max));
			}
		}
		return stimMod;
	}
	
	public EnumMap<Goods.Types, Double> nextStimModFood(double min, double max) {
		EnumMap<Goods.Types, Double> stimMod =  new EnumMap<Goods.Types, Double>(Goods.Types.class);
		for(Goods.Types t : Goods.Types.values()) {
			if(t.equals(Goods.Types.FOOD) || t.equals(Goods.Types.BELLS) || t.equals(Goods.Types.NONE)) {
				stimMod.put(t, randDouble(min, max));
			}
			else {
				stimMod.put(t, 0.0);
				//stimMod.put(t, randDouble(min, max));
			}
		}
		return stimMod;
	}
	
	public int[] randomBuildQueue() {
		//initialize build queue
		Integer[] fullQueue = new Integer[30];
		for(int i = 0; i<fullQueue.length; ++i) {
			fullQueue[i]=i;
		}
		//shuffle
		Collections.shuffle(Arrays.asList(fullQueue));
		//take the first 8
		int[] shortQueue = new int[gene2Size];
		for(int i = 0; i<gene2Size; ++i) {
			shortQueue[i] = fullQueue[i];
		}
		return shortQueue;
	}
	
	public void evaluate() {
		for(Gene e : this.currentGeneration) {
			e.setPriority(Thread.MIN_PRIORITY);
			e.start();
		}
		for(Gene e: this.currentGeneration) {
			try {
				e.join();
			} catch (InterruptedException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			}
		}
		int score = 0;
		for(Gene g: this.currentGeneration) {
			score += g.score;
		}
		System.out.println("Current Generation has been evaluated. Average score is "+score/generationSize);
		Collections.sort(this.currentGeneration);
		System.out.println("Best is "+this.currentGeneration.get(0));
	}
	
	public List<Gene> select() {
		List<Gene> selection = new ArrayList<Gene>();
		//save the best two individuals
		this.currentGeneration.add(best1);
		this.currentGeneration.add(best2);
		Collections.sort(this.currentGeneration);
		best1 = currentGeneration.get(0);
		best2 = currentGeneration.get(1);
		for(int i = 0; i < nrOfParents; ++i) {
			//selection.add(new Gene(this.currentGeneration.get(i).alpha, this.currentGeneration.get(i).stimuliModifiers, this.currentGeneration.get(i).buildQueue));
			selection.add(new Gene(this.currentGeneration.get(i)));
		}
		return selection;
	}
	
	public Gene CrossOver(Gene g1, Gene g2) {
				
		//crossover first chromosome
		int crossPoint = new Random().nextInt(gene1Size);
		
		Gene g = new Gene(g1);
		
		int i = 0;
		for(Goods.Types t: Goods.Types.values()) {
			if(i >= crossPoint) {
				g.stimuliModifiers.put(t, g2.stimuliModifiers.get(t));
			}
			++i;
			if(i >= gene1Size) break;
		}
		
		crossPoint = new Random().nextInt(gene1Size);
		i = 0;
		for(Goods.Types t: Goods.Types.values()) {
			if(i >= crossPoint) {
				g.stimModFood.put(t, g2.stimModFood.get(t));
			}
			++i;
			if(i >= gene1Size) break;
		}
		
		crossPoint = new Random().nextInt(gene1Size);
		i = 0;
		for(Goods.Types t: Goods.Types.values()) {
			if(i >= crossPoint) {
				g.stimModProd.put(t, g2.stimModProd.get(t));
			}
			++i;
			if(i >= gene1Size) break;
		}
		
		//crossover second chromosome
		int[] newBuildQueue = new int[8];
		crossPoint = new Random().nextInt(gene2Size);
		for(int j = 0; j < gene2Size; ++j) {
			int r = new Random().nextInt(2);
			if(r == 1) {
				newBuildQueue[j]=g1.buildQueue[j];
			}
			else {
				newBuildQueue[j]=g2.buildQueue[j];
			}
		}
		
		System.arraycopy(newBuildQueue, 0, g.buildQueue, 0, gene2Size);
		g.correctBuildQueue();
		
		//3rd chromosome
		//int crossPoint;
		//g = new Gene(g1);
		crossPoint = new Random().nextInt(gene3Size);
		if(crossPoint == 0) {
			g.beta = g2.beta;
			g.gamma = g2.gamma;
		}
		else {
			g.beta = g1.beta;
			g.gamma = g2.gamma;
		}
		
		return g;
	}
	
	public void mutate(Gene g) {
		
		//mutate first chromosome
		for(int i = 1; i < gene1Size+1; ++i) {
			if(new Random().nextDouble() <= mutationChance) {
				g.set(i, randDouble(minCoeff, maxCoeff));
			}
		}
		
		for(int i = 1; i < gene1Size+1; ++i) {
			if(new Random().nextDouble() <= mutationChance) {
				g.set(i, randDouble(minCoeff, maxCoeff), g.stimModFood);
			}
		}
		
		for(int i = 1; i < gene1Size+1; ++i) {
			if(new Random().nextDouble() <= mutationChance) {
				g.set(i, randDouble(minCoeff, maxCoeff), g.stimModProd);
			}
		}
		/*
		//mutate second chromosome
		for(int i = 0; i < gene2Size; ++i) {
			if(new Random().nextDouble() <= mutationChance) {
				g.buildQueue[i]=new Random().nextInt(50);
				g.correctBuildQueue();
			}
		}
		*/
		//mutate third chromosome
		if(new Random().nextDouble() <= mutationChance) {
			g.beta = randDouble(minCoeff2, maxCoeff2);
		}
		if(new Random().nextDouble() <= mutationChance) {
			g.gamma = randDouble(minCoeff2, maxCoeff2);
		}
	}
	
	public void nextGeneration() {
		//select best parents
		List<Gene> parents = select();
		List<Gene> children = new ArrayList<Gene>();
		Integer[] order = new Integer[parents.size()];
		for(int i = 0; i<parents.size(); ++i) {
			order[i]=i;
		}
		
		Collections.shuffle(Arrays.asList(order));
		//create children from randomly selected couples
		for(int i = 0; i < parents.size(); i +=2) {
			children.add(CrossOver(parents.get(order[i]), parents.get(order[i+1])));
			children.add(CrossOver(parents.get(order[i+1]), parents.get(order[i])));
		}
		//mutate
		this.nextGeneration.addAll(parents);
		this.nextGeneration.addAll(children);
		for(Gene g: this.nextGeneration) {
			mutate(g);
		}
		//make nextGeneration the current one
		currentGeneration.clear();
		currentGeneration.addAll(nextGeneration);
		nextGeneration.clear();
	}

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		// TODO Auto-generated method stub
		
		Game game = new Game();
		for(int i = 0; i < nrOfGeneration; ++i) {
			System.out.println("Generation "+i);
			game.evaluate();
			game.nextGeneration();
		}
	}

}
